MODULE MicroEnvModule                                                ! Compile after the modules below
   ! Latest revisions by MCN on November 1, 2011
   USE GlobalModule                                                  ! Compile this module first
   USE UtilityModule                                                 ! Compile this module second
   USE DistributionModule                                            ! Compile this module third

   IMPLICIT NONE                                                     ! Must declare all variables

   TYPE :: MICROENV                                                  ! Stores summary info for micros
      INTEGER  (KIND=IK)  :: Index                                   ! Position in the MIC vector
      CHARACTER(LEN=LEN2) :: Name                                    ! Name of micro (one word)
      CHARACTER(LEN=LEN3) :: Description                             ! Description of micro
      INTEGER  (KIND=IK)  :: Method                                  ! FACTORS (=1) or MASSBAL (=2)
      INTEGER  (KIND=IK)  :: PRX                                     ! MP index for proximity
      INTEGER  (KIND=IK)  :: PEN                                     ! MP index for penetration
      INTEGER  (KIND=IK)  :: AER                                     ! MP index for air exchange rate 
      INTEGER  (KIND=IK)  :: DEC                                     ! MP index for decay & deposition
      INTEGER  (KIND=IK)  :: VOL                                     ! MP index for volume
      INTEGER  (KIND=IK)  :: MR                                      ! MP index for mean removal rate
      INTEGER  (KIND=IK)  :: NES                                     ! # of emission sources 
      INTEGER  (KIND=IK)  :: NCS                                     ! # of concentration sources
    END TYPE MICROENV                                                ! Used to declare the MIC vector

   TYPE :: MICROPARAM                                                ! Stores properties of MP
      INTEGER (KIND=IK) :: Mic                                       ! Micro # to which MP belongs
      INTEGER (KIND=IK) :: Pol                                       ! Pollutant to which MP belongs
      INTEGER (KIND=IK) :: Par                                       ! Paramter type for this MP
      INTEGER (KIND=IK) :: Source                                    ! Source # for MIC (for products)
      INTEGER (KIND=IK) :: HB(24)                                    ! Hour - block mapping
      INTEGER (KIND=IK) :: WT(7)                                     ! Day of week - daytype mapping
      INTEGER (KIND=IK) :: MS(12)                                    ! Month - season mapping
      INTEGER (KIND=IK) :: C(3)                                      ! Conditional variable numbers 
      INTEGER (KIND=IK) :: N(7)                                      ! # of levels for each index
      INTEGER (KIND=IK) :: ResH                                      ! Resample hours (YES or NO) 
      INTEGER (KIND=IK) :: ResD                                      ! Resample days (YES or NO)
      INTEGER (KIND=IK) :: ResTS                                     ! Resample timesteps (YES or NO)
      INTEGER (KIND=IK) :: ResW                                      ! Resample workplace (YES or NO)
      INTEGER (KIND=IK) :: NShift                                    ! # of possible dayshift values
      INTEGER (KIND=IK) :: NDistrib                                  ! # distributions for this MP
      INTEGER (KIND=IK) :: ZDist                                     ! Offset for this MP in MPDistrib
      INTEGER (KIND=IK) :: CustomSeed = 0                            ! Initial custom seed for this MP
      INTEGER (KIND=IK) :: Seed(MAXK)                                ! Random number seed for this MP
      INTEGER (KIND=IK) :: HBR(24)                                   ! Hourly block repetition numbers
      INTEGER (KIND=IK) :: HBN(24)                                   ! Block sizes (# of hours)
      INTEGER (KIND=IK) :: CorrNum                                   ! Correlation subset number
   END TYPE MICROPARAM                                               ! Used to declare MP vector

   INTEGER (KIND=IK),  PARAMETER   :: FACTORS=1, MASSBAL=2           ! Codes for calculation types
   INTEGER (KIND=IK),  PARAMETER   :: PR=1,PE=2,AE=3,DE=4,VO=5       ! Codes for parameter types
   INTEGER (KIND=IK),  PARAMETER   :: MR=6,ES=7,CS=8                 ! More codes for paramter types
   INTEGER (KIND=IK),  ALLOCATABLE :: DA(:,:)                        ! District-Area map (MP,district)
   INTEGER (KIND=IK),  ALLOCATABLE :: TS(:,:)                        ! Daytype-Season(MP,day)
   INTEGER (KIND=IK),  ALLOCATABLE :: DayShift(:)                    ! CV indexing shift array (day)
   INTEGER (KIND=IK),  ALLOCATABLE :: NDist(:,:)                     ! Distribution # (hour,day) 
   INTEGER (KIND=IK),  ALLOCATABLE :: NSamp(:,:)                     ! Sample # (hour,day)
   INTEGER (KIND=IK),  ALLOCATABLE :: SampCount(:)                   ! # samples each distrib #)
   INTEGER (KIND=IK),  ALLOCATABLE :: NewDay(:)                      ! Newday flags for each dist
   INTEGER (KIND=IK),  ALLOCATABLE :: NES(:,:,:)                     ! terms per ES source (mic,source)
   INTEGER (KIND=IK),  ALLOCATABLE :: NCS(:,:,:)                     ! terms per CS source (mic,source) 
   TYPE (MICROENV),    ALLOCATABLE :: Mic(:,:)                       ! Micro properties (micro #)
   TYPE (MICROPARAM),  ALLOCATABLE :: MPar(:)                        ! MP descriptions (MP #)
   TYPE (CDIST),TARGET,ALLOCATABLE :: MPDistr(:)                     ! Array of MP Distribs (index #) 
   REAL (KIND=RK),     ALLOCATABLE :: EProd(:,:,:,:)                 ! EProducts (hr,mic,place,source)
   REAL (KIND=RK),     ALLOCATABLE :: CProd(:,:,:,:)                 ! CProducts (hr,mic,place,source)
   REAL (KIND=RK),     ALLOCATABLE :: ESum(:,:,:)                    ! Emission sums (hour,micro,place)
   REAL (KIND=RK),     ALLOCATABLE :: CSum(:,:,:)                    ! Conc source sums (",",")
   REAL (KIND=RK),     ALLOCATABLE :: MPV(:,:,:)                     ! MP values (timestep,MP,place)
   REAL (KIND=RK),     ALLOCATABLE :: PP(:,:)                        ! Prox * Penetration (hour,place)
   REAL (KIND=RK),     ALLOCATABLE :: RR(:)                          ! Removal rate (hour)
   REAL (KIND=RK),     ALLOCATABLE :: SS(:)                          ! Source strength (hour)
   REAL (KIND=RK),     ALLOCATABLE :: Buff(:,:)                      ! Distribution results (dist,hour) 
   REAL (KIND=RK),     ALLOCATABLE :: CVVals(:,:)                    ! Hourly CV values
   REAL (KIND=RK),     ALLOCATABLE :: MicroSeeds(:,:)                ! Custom seeds for each micro par 
 
CONTAINS                                                             ! Routines internal to this module

SUBROUTINE ReadMicroData                                             ! Called from main APEX program
   INTEGER   (KIND=IK)  :: Num, Val, NMP=0, A, B, D, LType, M, Total ! Local integer variables 
   INTEGER   (KIND=IK)  :: ZDist=0, DistN=0, I, K,Comp, J(7),N(7), W ! More local integer variables
   INTEGER   (KIND=IK)  :: MaxDist=0                                 ! Counter for array sizing
   INTEGER   (KIND=IK)  :: PL, NumSources=0, NumReq=0                ! Local integer variables
   INTEGER   (KIND=IK)  :: Seed(MAXK)                                ! Local integer variable  
   INTEGER   (KIND=IK), ALLOCATABLE :: AER_Defined(:),VOL_Defined(:) ! Local integer variables
   REAL      (KIND=RK), ALLOCATABLE :: R(:)                          ! Array for seeds
   CHARACTER (LEN=LEN4) :: String, UpString, SubS                    ! Buffers for input records
   CHARACTER (LEN=LEN1) :: Word2                                     ! Buffers for short words
   CHARACTER (LEN=LEN2) :: Word1                                     ! Buffers for micro name
   CHARACTER (LEN=1)    :: Char1                                     ! Buffer for a single character
   CHARACTER (LEN=14)   :: F1 = '(2X,I5,A23,I5)'                     ! Format for writing # sources
   CHARACTER (LEN=11)   :: F2 = '(2X,I5,A28)'                        ! Format for source zero count
   ! Global constants: IK, LEN1, LEN3, LogU, LV_FMT, LV_MSG, MicroU, NO, RK, ST_FMT, ST_MSG, YES
   ! Global vars used: Calen, CVList, DebugLevel, MPSeeds, NumCV, NumDays, NumMicros, NumSources
   ! Global vars set : Ios, MicroName, NumMP, NumMPDistrib, ProcLevel, ProcName
   ! Module constants: AE, CS, ES, FACTORS, MASSBAL, MR
   ! Module vars set : DA, Mic, MPar, MPDistr, NCS, NES, TS
   ! Intrinsic procs : ADJUSTL, ANY, CLOSE, IACHAR, INDEX, MAX, MAXVAL, PRODUCT,
   !                   READ, REWIND, TRIM, WRITE
   ! APEX procedures : Fatal, GetLine, Distribution, OpenInput, SetupMicArrays, Up, Warning
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadMicroData"                             ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(MicroU,h=NO)                                       ! Open microenvironment input file
   ALLOCATE (AER_Defined(NumMicros),STAT=AllocErr)                   ! Make space for AER flags
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (VOL_Defined(NumMicros),STAT=AllocErr)                   ! Make space for VOL flags
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (R(NumProfiles),STAT=AllocErr)                           ! Make space for custom seed vals
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   NumMP = 0                                                         ! Initialize MicroParam counter
   NumMPDistrib = 0                                                  ! Initialize distrib counter
   DO                                                                ! Loop over input records
      CALL GetLine(MicroU, String, LType)                            ! Get one record from input file  
      IF (Ios(MicroU)<0) EXIT                                        ! Exit loop on end-of-file
      UpString=Up(String)                                            ! Make uppercase version of line
      IF (UpString(1:3)=="PAR") THEN                                 ! Each MP must have a keyword PAR
         NumMP = NumMP+1                                             ! Increment MP counter
         DistN = 0                                                   ! Start distrib counter for MP
      ENDIF                                                          ! End of new MP logic block
      IF (NumMP>0.AND.INDEX("1234567890",UpString(1:1))>0) THEN      ! If MP and line is numeric...
         DistN = DistN + 1                                           ! Then it must be a distribution
         NumMPDistrib = NumMPDistrib + 1                             ! Increment both counters
         IF (MaxDist<DistN) MaxDist = DistN                          ! Reset maximum if necessary    
      ENDIF                                                          ! End of distribution counting
   ENDDO                                                             ! Continue with next record 
   REWIND(MicroU)                                                    ! Go back to start of input file
   CALL SetupMicArrays(MaxDist)                                      ! Code is here in MicroEnvModule
   MPar(:)%Mic    = 0                                                ! Undefined MP have no micro
   MPar(:)%Par    = 0                                                ! Undefined MP have no param type
   MPar(:)%Source = 0                                                ! Undefined MP have no source #
   DO I=1,NumMP                                                      ! Loop over MP vector
      MPar(I)%HB(:)   = 1                                            ! Default block is 1 for all hours
      MPar(I)%WT(:)   = 1                                            ! Default daytype is 1 
      MPar(I)%MS(:)   = 1                                            ! Default season is 1 
      MPar(I)%C(:)    = 0                                            ! Default conditional var is none
      MPar(I)%N(:)    = 1                                            ! Default # distribs is 1
      MPar(I)%HBR(:)  = 1                                            ! Default hour repetition is 1
      MPar(I)%HBN(:)  = 1                                            ! Default block count is 1
   ENDDO                                                             ! Continue with next MP
   MPar(:)%ResH = NO                                                 ! Default is no hourly resampling
   MPar(:)%ResTS= NO                                                 ! Default is no timestep resamp
   MPar(:)%ResD = NO                                                 ! Default is no daily resampling
   MPar(:)%ResW = YES                                                ! Default is work resampling
   AER_Defined = NO                                                  ! Initialize AER flags
   VOL_Defined = NO                                                  ! Initialize VOL flags
   DistN = 0                                                         ! Initialize distribution counter
   NMP   = 0                                                         ! Initialize MP counter
   PL   = 0                                                          ! Initialize pollutant counter
   DO                                                                ! Loop over input records  
      MEFlag = 1                                                     ! Turns on line counting in GetLine
      CALL GetLine(MicroU, String, LType)                            ! Get one line from input file
      IF (Ios(MicroU)<0) EXIT                                        ! Exit loop on end-of-file
      UpString = Up(String)                                          ! Make uppercase version of line 
      IF (UpString(1:12)=="MICRO NUMBER") NMP = NMP + 1              ! Each MP starts with this key
      IF (NMP==0.AND.INDEX("1234567890",UpString(1:1))>0) THEN       ! This case is for micro data
         READ(String,*) Num, Word1, Word2                            ! Get micro number, name, method
         Word2 = Up(Word2)                                           ! Convert method to uppercase
         IF (Num<1 .OR.Num>127) CALL Fatal(1)                        ! Allowed range of micro # (1-127)
         IF (Num>NumMicros)     CALL Fatal(2)                        ! Micro # limited by PARAMS file
         IF (ANY(Mic(:,Num)%Index>0))  CALL Warning(1)               ! Micro should not be listed twice
         I=MAX(INDEX(UpString,"MASSBAL"),INDEX(UpString,"FACTORS"))  ! Find keyword for method
         IF (I==0)              CALL Fatal(3)                        ! Stop run if method not found
         Mic(:,Num)%Index  = Num                                     ! Set micro index
         Mic(:,Num)%Name   = Word1                                   ! Set micro name
         MicroName(Num)  = Word1                                     ! Set global micro name
         IF (Word2(1:1)=="M") Mic(:,Num)%Method = MASSBAL            ! Set method to MASSBAL
         IF (Word2(1:1)=="F") Mic(:,Num)%Method = FACTORS            ! Set method to FACTORS
         Mic(:,Num)%Description = ADJUSTL(UpString(I+7:))            ! Beyond method is description
      ELSE IF (NMP==0) THEN                                          ! This is non-data, non-MP line
         CYCLE                                                       ! Skip this input record
      ELSE                                                           ! Must be in MP data section
         SubS = ADJUSTL(UpString(INDEX(UpString,"=")+1:))            ! Data for keyword lines
         SELECT CASE(UpString(1:5))                                  ! Identify keyword using 5 chars
         CASE("MICRO")                                               ! Keyword for micro number 
            READ(SubS,*) Val                                         ! Extract micro # from input line
            IF (Val<1.OR.Val>127)    CALL Fatal(1)                   ! Stop if micro # out of range
            IF (Mic(1,Val)%Index/=Val) CALL Fatal(4)                 ! Stop if micro not defined
            MPar(NMP)%Mic   = Val                                    ! Set micro # for this MP
            ZDist = ZDist + DistN                                    ! Update ZDist by last MP count
            MPar(NMP)%ZDist = ZDist                                  ! Set Zdist for this MP
            IF (DistN < NumReq ) CALL Fatal(5)                       ! Not enough dists for prev MP
            DistN = 0                                                ! Reset distrib counter for MP
            PL    = 0                                                ! Reset pollutant number
         CASE("CORRN")                                               ! Keyword for correlation set
            READ(SubS,*) Val                                         ! Extract correlation # from input 
            MPar(NMP)%CorrNum   = Val                                ! Set corr # for this MP
            IF (Val>MaxCorrNum) MaxCorrNum=Val                       ! Update the maximum corr subset #
         CASE("POLLU")                                               ! Keyword for parameter type
            READ(SubS,*) PL                                          ! Extract pol # from input line
            IF (PL<1.OR.PL>NumPollutants)  CALL Fatal(6)             ! Stop if pollutant# out of range
            MPar(NMP)%Pol   = PL                                     ! Set pollutant # for this MP
            NumSources      = Pol(PL)%NumSources                     ! Number of sources for pollutant
         CASE("PARAM")                                               ! Keyword for parameter type
            Val = (INDEX("PRPEAEDEVOMRESCS",SubS(1:2))+1)/2          ! Find parameter type in this list
            IF (PL==0 .AND. (Val .NE. 3) .AND.            &   
               (Val .NE. 5)  )   CALL Fatal(38)                      ! Missing pollutant number for MP
            IF (SubS(1:2)=="AI") Val=AE                              ! Alternate form for 'AE'
            IF (SubS(1:2)=="ME") Val=MR                              ! Alternate form for 'MR'
            IF (SubS(1:2)=="EM") Val=ES                              ! Alternate form for 'ES'
            IF (SubS(1:2)=="CO") Val=CS                              ! Alternate form for 'CS'
            IF(Val==0) CALL Fatal(7)                                 ! Unknown parameter type
            MPar(NMP)%Par = Val                                      ! Set parameter type for this MP
            M = MPar(NMP)%Mic                                        ! Micro for this MP
            IF (Val==1) Mic(PL,M)%PRX = NMP                          ! Map micro(PL,M) proximity to MP
            IF (Val==2) Mic(PL,M)%PEN = NMP                          ! Map micro(PL,M) penetration to MP
            IF (Val==3) THEN                                         ! If param is air exchange rate
               IF (AER_Defined(M)==YES)  CALL Fatal(8)               ! AER for micro already defined
               DO I = 1, NumPollutants                               ! All pollutants get same AER           
                  Mic(I,M)%AER = NMP                                 ! Map micro(PL,M) air exch. to MP
               ENDDO                                                 ! End loop over pollutants
               AER_Defined(M)=YES                                    ! Update AER flag
            ENDIF                                                    ! End AER logic
            IF (Val==4) Mic(PL,M)%DEC = NMP                          ! Map micro(PL,M) decay rate to MP
            IF (Val==5) THEN                                         ! If parmeter is volume  
               IF (VOL_Defined(M)==YES)  CALL Fatal(9)               ! Vol for micro already defined
               DO I = 1, NumPollutants                               ! All pollutants get same VOL           
                  Mic(I,M)%VOL = NMP                                 ! Map micro(PL,M) vol to MP              
               ENDDO                                                 ! End loop over pollutants
               VOL_Defined(M)=YES                                    ! Update VOL flag
            ENDIF                                                    ! End VOL logic
            IF (Val==6) Mic(PL,M)%MR  = NMP                          ! Map micro(PL,M) removal rate to MP
            IF (Val==7) Mic(PL,M)%NES = Mic(PL,M)%NES + 1            ! Increment micro(PL,M) ES counter
            IF (Val==8) Mic(PL,M)%NCS = Mic(PL,M)%NCS + 1            ! Increment micro(PL,M) CS counter
         CASE("SOURC")                                               ! Keyword for source number
            READ(SubS,*) Val                                         ! Extract source # from input line
            IF (MPar(NMP)%Par==ES.OR.MPar(NMP)%Par==CS) THEN         ! Check for source parameter type
               IF (Val<0 .OR. Val>NumSources) CALL Fatal(10)         ! Stop if source # out of range
               MPar(NMP)%Source = Val                                ! Set source number for this MP
            ENDIF                                                    ! End of ES or CS logic block 
         CASE("HOURS")                                               ! Keyword for hours-blocks mapping
            READ(SubS,*,IOSTAT=Ios(MicroU)) MPar(NMP)%HB             ! Read 24 values from input
            IF (Ios(MicroU)/=0) CALL Fatal(11)                       ! Stop if trouble reading values
            IF (ANY(MPar(NMP)%HB<1)) CALL Fatal(12)                  ! Stop if block # out of range
            MPar(NMP)%N(1) = MAXVAL(MPar(NMP)%HB)                    ! # blocks is largest of 24 values
         CASE("WEEKD")                                               ! Keyword for weekday-daytype map
            READ(SubS,*,IOSTAT=Ios(MicroU)) MPar(NMP)%WT             ! Read 7 values from input
            IF (Ios(MicroU)/=0) CALL Fatal(13)                       ! Stop if trouble reading values
            IF (ANY(MPar(NMP)%WT<1)) CALL Fatal(14)                  ! Stop if daytype # out of range
            MPar(NMP)%N(2) = MAXVAL(MPar(NMP)%WT)                    ! # daytypes is largest value
         CASE("MONTH")                                               ! Keyword for Month-Season mapping
            READ(SubS,*,IOSTAT=Ios(MicroU)) MPar(NMP)%MS             ! Read 12 values from input
            IF (Ios(MicroU)/=0) CALL Fatal(15)                       ! Stop if trouble reading values
            IF (ANY(MPar(NMP)%MS<1)) CALL Fatal(16)                  ! Stop if Season # out of range
            MPar(NMP)%N(3) = MAXVAL(MPar(NMP)%MS)                    ! # seasons is largest value
         CASE("DISTR")                                               ! Keyword for District-Area map
            READ(SubS,*,IOSTAT=Ios(MicroU)) DA(NMP,:)                ! Read (#districts) values 
            IF (Ios(MicroU)/=0) CALL Fatal(17)                       ! Stop if trouble reading values
            IF (ANY(DA(NMP,:)<1)) CALL Fatal(18)                     ! Stop if Area is out of range
            MPar(NMP)%N(4) = MAXVAL(DA(NMP,:))                       ! # Areas is largest value 
      CASE("CONDI")                                                  ! Keyword for conditional variable
            Val = INDEX(UpString,"#")                                ! Locate pound sign in input line
            IF (Val==0) CALL Fatal(19)                               ! Stop if no pound sign found
            READ(UpString(Val+1:),*,IOSTAT=Ios(MicroU)) Num          ! Get condition number
            IF (Ios(MicroU)/=0) CALL Fatal(20)                       ! Stop if CV number is unreadable 
            IF (Num<1.OR.Num>3) CALL Fatal(21)                       ! Stop if CV number out of range
            DO I=1,NumCV                                             ! Loop over CV List
               IF (TRIM(CVList(I)%Name)==TRIM(SubS)) EXIT            ! Exit loop if name of CV found
            ENDDO                                                    ! Continue with next CV 
            IF (TRIM(Up(SubS))=="FACTORGROUP" .AND. NumLevels>0) THEN ! FactorGroup handled uniquely
               CVlist(13)%Supplied=YES                               ! It is supplied
               CVList(13)%Levels=NumLevels                           ! Assigning levels
            ENDIF                                                    ! End factor group logic
            IF (I>NumCV) THEN                                        ! If no CV match is found...
               IF (TRIM(SubS) .NE. "0" .AND. TRIM(SubS).NE."")      &  
                  Call Fatal(22)                                     ! Invalid conditional variable    
               MPar(NMP)%C(Num)   = 0                                ! No CV used for this condition
               MPar(NMP)%N(4+Num) = 1                                ! One level by default
            ELSE                                                     ! Else CV match found
               MPar(NMP)%C(Num)   = I                                ! Identify CV by number
               CVList(I)%Required = YES                              ! This CV must be defined
               IF (CVlist(I)%Supplied==NO) CALL Fatal(23)            ! Stop if CV is not defined
               MPar(NMP)%N(4+Num) = CVList(I)%Levels                 ! Set # levels for the CV
               IF (DebugLevel>1) THEN                                ! If user wants details
                  WRITE(LogU,*) 'Cond #',Num, '=', I                 ! Report the CV match
                  WRITE(LogU,*) 'Levels=', MPar(NMP)%N(4+Num)        ! Debugging message
               ENDIF                                                 ! End of debug block
            ENDIF                                                    ! End of logic block for CV match
         CASE("RESAM")                                               ! Keyword for Resampling options
            Val = NO                                                 ! Default to NO
            IF (Subs(1:1)=="Y") Val= YES                             ! Set to YES if 'Y' found
            IF (UpString(7:7)=="H") MPar(NMP)%ResH = Val             ! Character 7 (right  
            IF (UpString(7:7)=="D") MPar(NMP)%ResD = Val             ! after 'Resamp')  
            IF (UpString(7:7)=="W") MPar(NMP)%ResW = Val             ! distinguishes  
            IF (UpString(7:7)=="T") MPar(NMP)%ResTS= Val             ! the 4 options.
         CASE("RANDO")                                               ! Keyword for RandomSeed
            READ(SubS,*,IOSTAT=Ios(MicroU)) Val                      ! Value to right of '='
            IF (Ios(MicroU)/=0) CALL Fatal(24)                       ! Stop if value is unreadable
            IF (Val>0) MPar(NMP)%CustomSeed = Val                    ! Set init randseed for this MP
            Seed(1:KSEED)=MPar(NMP)%CustomSeed;                      ! Temp seed variable   
            CALL RANDOM_SEED(Put=Seed(1:KSeed))                      ! Initialize seed
            CALL RANDOM_NUMBER(R)                                    ! Generate seeds for all people
            MicroSeeds(:,NMP) = R                                    ! Save the seeds for use           
         CASE("BLOCK")                                               ! Keyword for start of distribs
            MPar(NMP)%NDistrib = PRODUCT(MPar(NMP)%N(:))             ! Calculate # of distribs
            NumReq = MPar(NMP)%NDistrib                              ! Number of required dists
            MPar(NMP)%NShift   = PRODUCT(MPar(NMP)%N(4:7))           ! Set # dayshift values
         CASE DEFAULT                                                ! No more keywords 
            Num = IACHAR(UpString(1:1))                              ! Convert first character to ASCII                                             
            IF (Num>48.AND.Num<58) THEN                              ! If it is a digit 1-9 ...
               N = MPar(NMP)%N                                       ! Local vector for distr. indices 
               READ(UpString,*,IOSTAT=IOS(MicroU)) J, Char1          ! Read 7 indices and a shape 
               IF (Ios(MicroU)/=0) CALL Fatal(25)                    ! Stop if trouble reading data
               IF (ANY(J<1)) CALL Fatal(26)                          ! Stop if index not positive
               IF (ANY(J-N>0)) CALL Fatal(27)                        ! Stop if index too large
               DistN = DistN + 1                                     ! Increment distribution counter            
               IF (DistN>MPar(NMP)%NDistrib) CALL Fatal(28)          ! Stop if too many distribs
               J = J-1                                               ! Start all indices at zero
               I = J(4)+N(4)*(J(5)+N(5)*(J(6)+N(6)*J(7)))            ! Locate distribution in order
               I = 1+J(1)+N(1)*(J(2)+N(2)*(J(3)+N(3)*I))             !    according to indices
               IF (I<0.OR.I>MPar(NMP)%NDistrib) CALL Fatal(29)       ! Check range on distrib index
               I = ZDist + I                                         ! Find position in full list    
               IF (MPDistr(I)%Shape>0) CALL Fatal(30)                ! Stop if distrib already defined
               UpString = UpString(INDEX(UpString,Char1):)           ! Drop indices and P0 from line
               MPDistr(I) = ReadDist(UpString)                       ! Read distribution parameters
            ELSE                                                     ! Else not a numeric line
               CALL Fatal(31)                                        ! Then stop
            ENDIF                                                    ! End of CASE DEFAULT logic block
         END SELECT                                                  ! End of SELECT logic
      ENDIF                                                          ! End of GetLine processing logic
   ENDDO                                                             ! Continue with next input record
   CLOSE(MicroU)                                                     ! Close micro descriptions file
   MEFlag = 0                                                        ! Turns off counting in GetLine
   WRITE(LogU,*) 'Finished reading micro file'                       ! Status message for log file
   IF (ANY(MPDistr%Shape==0)) CALL Fatal(32)                         ! All distribs should be defined
   IF (NMP /= NumMP) CALL Fatal(33)                                  ! Running counter should = total
   IF (NumMPDistrib /= ZDist + DistN) CALL Fatal(34)                 ! Running counter should = total
   DO I=1, NumMP                                                     ! Loop over MP 
      IF (MPar(I)%Par==ES) THEN                                      ! If MP is an emissions source...
         M = MPar(I)%Mic                                             ! Find the micro it belongs to 
         A = MPar(I)%Source                                          ! Find the source number
         PL = MPar(I)%Pol                                            ! Find the pollutants number   
         NES(PL,M,A) = NES(PL,M,A)+1                                 ! Increment the source counter 
      ENDIF                                                          ! End of ES logic block 
      IF (MPar(I)%Par==CS) THEN                                      ! If MP is a concentration source
         M = MPar(I)%Mic                                             ! Find the micro it belongs to 
         A = MPar(I)%Source                                          ! Find the source number
         PL = MPar(I)%Pol                                            ! Find the pollutants number   
         NCS(PL,M,A) = NCS(PL,M,A)+1                                 ! Increment the source counter 
      ENDIF                                                          ! End of CS logic block 
   ENDDO                                                             ! Continue with next MP
   DO I=1,MaxCorrNum                                                 ! Loop over correlation subsets
     Comp=0                                                          ! Init index of MPs to compare
     DO K=1, NMP                                                     ! Loop over all micro parameters    
        IF (MPar(K)%CorrNum==I .AND. Comp>0) THEN                    ! If need to compare settings
           IF (ANY(MPar(K)%HB-MPar(Comp)%HB.ne.0)) CALL Warning(2)   ! Compare hour-block mapping       
           IF (ANY(MPar(K)%WT-MPar(Comp)%WT.ne.0)) CALL Warning(2)   ! Compare day-daytype mapping       
           IF (ANY(MPar(K)%MS-MPar(Comp)%MS.ne.0)) CALL Warning(2)   ! Compare month-season mapping       
           IF (ANY(MPar(K)%C -MPar(Comp)%C .ne.0)) CALL Warning(3)   ! Compare consitional vars       
           IF (MPar(K)%ResH .ne. MPar(Comp)%ResH)  CALL Warning(4)   ! Compare resampling      
           IF (MPar(K)%ResD .ne. MPar(Comp)%ResD)  CALL Warning(4)   ! Compare resampling      
           IF (MPar(K)%ResTS.ne. MPar(Comp)%ResTS) CALL Warning(4)   ! Compare resampling       
           IF (MPar(K)%ResW .ne. MPar(Comp)%ResW)  CALL Warning(4)   ! Compare resampling         
        ENDIF                                                        ! End comparison logic       
        IF (MPar(K)%CorrNum==I)  Comp=K                              ! Update comparison index   
     ENDDO                                                           ! End loop over micropars   
   ENDDO                                                             ! End loop over corr subsets
   WRITE(LogU,'(/A27)') " Source terms in each micro"                ! Put header on log file
   DO PL=1, NumPollutants                                            ! Loop over pollutants
      WRITE(LogU,'(1X,A13,A40)')                                    &
            " Pollutant = ",Pol(PL)%Name                             ! Report pol number and name
       DO M=1,NumMicros                                              ! Loop over micros
         WRITE(LogU,'(3X,A13,I5,2X,A40)')                           &
            " Micro number",M,MicroName(M)                           ! Report micro number and name
         Total = 0                                                   ! Total number of sources
         A = NES(PL,M,0)                                             ! # unnembered ES sources   
         IF(A==1) WRITE(LogU,F2) A," ES source  put in  #     0"     ! Report ES #0 count to log file
         IF(A>1)  WRITE(LogU,F2) A," ES sources put in  #     0"     ! Report ES #0 count to log file
         B = NCS(PL,M,0)                                             ! # unnumbered CS sources  
         IF(B==1) WRITE(LogU,F2) B," CS source  put in  #     0"     ! Report CS #0 count to log file
         IF(B>1)  WRITE(LogU,F2) B," CS sources put in  #     0"     ! Report CS #0 count to log file
         DO I=1, NumSources                                          ! Loop over nonzero source #s
            A = NES(PL,M,I)                                          ! Get count of ES sources
            B = NCS(PL,M,I)                                          ! Get count of CS sources 
            IF (A>0.AND.B>0) CALL Fatal(35)                          ! Stop if source # is both ES & CS
            IF (A==1) WRITE(LogU,F1) A," term  for ES source # ",I   ! Report # of terms for ES source
            IF (A>1)  WRITE(LogU,F1) A," terms for ES source # ",I   ! Report # of terms for ES source
            IF (B==1) WRITE(LogU,F1) B," term  for CS source # ",I   ! Report # terms for CS source
            IF (B>1)  WRITE(LogU,F1) B," terms for CS source # ",I   ! Report # terms for CS source
            IF (A>0.OR.B>0) Total = Total + 1                        ! Increment source counter
         ENDDO                                                       ! Continue with next source #
         WRITE(LogU,*)                                              &
            "     Total # sources =",Total+NES(PL,M,0)+NCS(PL,M,0)   ! Write total # sources for micro
         IF (Mic(PL,M)%Method==MASSBAL) THEN                         ! For MASSBAL micros...
            IF (Mic(PL,M)%AER<=0) CALL Fatal(36)                     ! Stop if AER is not defined
            IF (Mic(PL,M)%NES>0.AND.Mic(PL,M)%VOL<=0) CALL Fatal(37) ! Stop if ESource but no volume
         ENDIF                                                       ! End of MASSBAL logic 
      ENDDO                                                          ! Continue with next micro
   ENDDO                                                             ! Continue with next pollutant
   ALLOCATE (MPSeeds(NumMP),STAT=AllocErr)                           ! Allocate space for MP seeds
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   DO I=1,NumMP                                                      ! Loop over MPs
      DO D=1,NumDays                                                 ! Loop over simulation days
         W = MPar(I)%WT(Calen(D)%DayOfWeek)-1                        ! Find Daytype for day and MP
         TS(I,D) = W*MPar(I)%N(2)+ MPar(I)%MS(Calen(D)%Month)        ! Set Daytype-Season combination
      ENDDO                                                          ! Continue with next day
      WRITE(LogU,'(/A15,I3)')  " Data for MP # ",I                   ! Write info on each MP to log
      WRITE(LogU,'(A15,I3)')   " Pollutant # = ",MPar(I)%Pol         ! Pollutant number
      WRITE(LogU,'(A15,I3)')   " Micro #     = ",MPar(I)%Mic         ! Micro number
      WRITE(LogU,'(A15,I3)')   " Param Type  = ",MPar(I)%Par         ! Parameter type
      WRITE(LogU,'(A15,I3)')   " Source #    = ",MPar(I)%Source      ! Source number
      WRITE(LogU,'(A15,24I3)') " Hour-Block  = ",MPar(I)%HB          ! Hour-Block mapping
      WRITE(LogU,'(A15,7I3)')  " Weekday-DT  = ",MPar(I)%WT          ! Weekday-Daytype mapping
      WRITE(LogU,'(A15,12I3)') " Month-Season= ",MPar(I)%MS          ! Month-Season mapping
      WRITE(LogU,'(A15,3I3)')  " Conditionals= ",MPar(I)%C           ! Conditional variable numbers
      WRITE(LogU,'(A15,7I3)')  " # levels    = ",MPar(I)%N           ! Number of levels per dimension
      WRITE(LogU,'(A15,A3)')   " ResampHours = ",YESNO(MPar(I)%ResH) ! Resample hours (YES or NO) 
      WRITE(LogU,'(A15,A3)')   " ResampTS    = ",YESNO(MPar(I)%ResTS)! Resample hours (YES or NO) 
      WRITE(LogU,'(A15,A3)')   " ResampDays  = ",YESNO(MPar(I)%ResD) ! Resample days (YES or NO)
      WRITE(LogU,'(A15,A3)')   " ResampWork  = ",YESNO(MPar(I)%ResW) ! Resample workplace (YES or NO)
      WRITE(LogU,'(A15,I3)')   " NShift      = ",MPar(I)%NShift      ! # places to shift distrib list
      WRITE(LogU,'(A15,I3)')   " Ndistrib    = ",MPar(I)%NDistrib    ! # of distributions needed 
      WRITE(LogU,'(A15,I3)')   " ZDist       = ",MPar(I)%Zdist       ! Offset in distribution list
      WRITE(LogU,'(A15,4I11)') " RandomSeed  = ",MPar(I)%Seed(1:4)   ! Random seed for this MP
   ENDDO                                                             ! Continue with next MP
   WRITE(LogU,*)                                                     ! Put blank line on log
   WRITE(LogU,*) "            MPD#    Shape     Par1       Par2  "  &
      //"     Par3     Par4         LTrunc    UTrunc    ResampOut"   ! Header for MP data  
   DO I=1, NumMPDistrib                                              ! Loop over MP definitions
      WRITE(LogU,'(A10,I6,A10,6F11.4,A6)')'MPDistrib',I,            &
      DistText(MPDistr(I)%Shape),MPDistr(I)%P1,                     &
      MPDistr(I)%P2,MPDistr(I)%P3, MPDistr(I)%P4,MPDistr(I)%LTrunc, &
      MPDistr(I)%UTrunc, YESNO(MPDistr(I)%ResampOut)                 ! Write MP info to log file
   ENDDO                                                             ! Continue with next MP  
   DEALLOCATE (AER_Defined, VOL_Defined, R)                          ! Deallocate internal arrays
   WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)                    ! Status message for log file
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadMicroData                                         ! Return to main APEX3 program

SUBROUTINE SetupMicArrays(MaxDist)                                   ! Called from ReadMicroData
   INTEGER (KIND=IK),INTENT(IN) :: MaxDist                           ! Input arguments
   INTEGER (KIND=IK)            :: NumSources                        ! Local integer variables
   ! Global constants: IK, LogU, LV_MS2, NO, SL_FM2, ST_MS2, YES
   ! Global vars used: DebugLevel, KSeed, NumDays, NumDistricts, NumHours, NumMicros, NumMP,
   !                   NumMPDistrib, NumSources
   ! Global vars set : MicroConc, MicroName, ProcLevel, ProcName
   ! Module vars set : Buff, CProd, CSum, DA, DayShift, DCN, EProd, ESum, Mic, MPar, 
   !                   MPDistr, MPV, NCS, NDist, NES, NSamp, PP, RR, SS, TS
   ! Intrinsic procs : ALLOCATE, WRITE
   ! APEX procedures : CheckAllocation, Fatal, Warning
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "SetupMicArrays"                            ! Procedure name for messages 
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   IF (NumMicros==0)    CALL Fatal(1)                                ! Stop if no micros defined  
   IF (NumDistricts==0) CALL Fatal(2)                                ! Stop if no air quality districts
   IF (NumMP==0)        CALL Warning(1)                              ! Warn user if no MP defined
   IF (NumMPDistrib==0) CALL Warning(2)                              ! Warn user if no MP distribs used
   IF (RoadWayFlag==YES)    NumLocs = 4                              ! If using RoadAQ
   IF (UseNHLocs==YES)      NumLocs = 5                              ! If using near home locs  
   IF (UseNWLocs==YES)      NumLocs = 6                              ! If using near work locs
   IF (RoadLastFlag==YES)   NumLocs = 7                              ! If using RoadAQLast
   NumSources = MAXVAL(Pol(:)%NumSources)                            ! Maximum number of sources 
   WRITE(LogU,*) '   # Micro-Parameters = ', NumMP                   ! Report # MP to log file
   WRITE(LogU,*) '   # MP Distributions = ', NumMPDistrib            ! Report # MP distribs to log
   WRITE(LogU,*) '   # Micros           = ', NumMicros               ! Report # micros to log
   WRITE(LogU,*) '   # Air Districts    = ', NumDistricts            ! Report # AQ districts to log
   ALLOCATE (MicroConc((-1*TimestepsPerDay+1):NumTS,0:NumMicros,    &
      NumLocs),STAT=AllocErr)                                        ! Space for micro conc values
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (MicroName(0:NumMicros),STAT=AllocErr)                   ! Space for global micro names
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (Mic(NumPollutants,NumMicros),STAT=AllocErr)             ! Space for properties of micros
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (MPDistr(NumMPDistrib),STAT=AllocErr)                    ! Space for MP distributions
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (MPar(NumMP),STAT=AllocErr)                              ! Space for MP descriptions
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (DA(NumMP,-1:NumDistricts),STAT=AllocErr)                ! Space for district-area mappings
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TS(NumMP,NumDays),STAT=AllocErr)                        ! Space for daytype-season indices
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (Buff(MaxDist,NumTS),STAT=AllocErr)                      ! Buffer for generated random #s
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (SampCount(MaxDist),STAT=AllocErr)                       ! Space for samp counts for dists 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (NewDay(MaxDist),STAT=AllocErr)                          ! Space for newday counts for dists 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (DayShift(NumDays),STAT=AllocErr)                        ! Space for day category results 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (NDist(TimeStepsPerDay,NumDays),STAT=AllocErr)           ! Space for timestep-dist  map
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (NSamp(TimeStepsPerDay,NumDays),STAT=AllocErr)           ! Space for timestep-sample# map
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (CVVals(7,NumTS),STAT=AllocErr)                          ! Space for CV values
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (NES(NumPollutants,NumMicros,0:NumSources),STAT=AllocErr)! Space for ES source counts
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (NCS(NumPollutants,NumMicros,0:NumSources),STAT=AllocErr)! Space for CS source counts
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (EProd((-1*TimestepsPerDay+1):NumTS,NumMicros,NumLocs,  & 
      NumSources),STAT=AllocErr)                                     ! Space for ESource products
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (CProd((-1*TimestepsPerDay+1):NumTS,NumMicros,NumLocs,  &
      NumSources), STAT=AllocErr)                                    ! Space for CSource products
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (ESum((-1*TimestepsPerDay+1):NumTS,NumMicros,NumLocs),  &
      STAT=AllocErr)                                                 ! Space for ESource sums
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (CSum((-1*TimestepsPerDay+1):NumTS,NumMicros,NumLocs),  &
      STAT=AllocErr)                                                 ! Space for CSource sums
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (MPV((-1*TimestepsPerDay+1):NumTS,-2:NumMP,NumLocs),    &  
      STAT=AllocErr)                                                 ! Space for generated MP values
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (PP((-1*TimestepsPerDay+1):NumTS,NumLocs),STAT=AllocErr) ! Space for (prox*pene) product
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (RR((-1*TimestepsPerDay+1):NumTS),STAT=AllocErr)         ! Space for hourly removal rates
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (SS((-1*TimestepsPerDay+1):NumTS),STAT=AllocErr)         ! Space for hourly source strength
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (MicroSeeds(NumProfiles, NumMP),STAT=AllocErr)           ! Space for custom seeds
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   IF (UseNWLocs==YES) THEN                                          ! If near work used
      ALLOCATE (NWLocs(-1*TimeStepsPerDay+1:NumTS),STAT=AllocErr)    ! Space for near work locs
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
   ENDIF                                                             ! End near work logic
   IF (UseNHLocs==YES) THEN                                          ! If near home used
      ALLOCATE (NHLocs(-1*TimeStepsPerDay+1:NumTS),STAT=AllocErr)    ! Space for near home locs
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
   ENDIF                                                             ! End near home logic
   MicroConc(:,0,:) = 0.                                             ! Micro #0 has zero conc
   MicroName(0)   = "ZeroExposure"                                   ! Name for micro #0
   Mic(:,:)%Index   = 0                                              ! Undefined micros have no index
   Mic(:,:)%Method  = 0                                              ! Undefined micros have no method
   Mic(:,:)%PRX     = -1                                             ! Default prox is MP=-1 (all ones)
   Mic(:,:)%PEN     = -1                                             ! Default pene is MP=-1 (all ones)
   Mic(:,:)%DEC     = 0                                              ! Default decay is zero
   Mic(:,:)%AER     = 0                                              ! Default AER = 0
   Mic(:,:)%VOL     = 0                                              ! Setting a default VOL (M)
   Mic(:,:)%MR      = -2                                             ! Default removal is MP=-2
   Mic(:,:)%NES     = 0                                              ! Default # ESource is zero
   Mic(:,:)%NCS     = 0                                              ! Default # CSource is zero
   NES(:,:,:)       = 0                                              ! Zero # terms for each ES source 
   NCS(:,:,:)       = 0                                              ! Zero # terms for each CS source 
   MPDistr(:)%Shape = 0                                              ! Undefined shapes are zero
   DA(:,:)        = 1                                                ! Default area is 1 
   MPV(:,-2,:)    = 1.                                               ! Initial MR value is 1.
   MPV(:,-1,:)    = 1.                                               ! MP=-1 stays at value=1 
   MPV(:,0,:)     = 0.                                               ! MP=0  stays at value=0
   IF (DebugLevel>0) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE SetupMicArrays                                        ! Return to ReadMicroData

SUBROUTINE MicroConcs(P,PL)                                          ! Called from main APEX program
   INTEGER (KIND=IK),INTENT(IN) :: P, PL                             ! Input arguments(profile,pollut)
   TYPE(CDist)       :: D                                            ! Local var for distributions
   INTEGER (KIND=IK) :: LP(7), MP, M, S, DSS, Step                   ! Local integer variables
   INTEGER (KIND=IK) :: LL, L, T, K, I, J, DNum, Dis, H              ! More local integer variables
   INTEGER (KIND=IK) :: Resamp, ResampH, ResampD, TS, Hour, LHour    ! More local integer variables
   CHARACTER(LEN=LEN2):: Name                                        ! Buffer for dist and site names    
   INTEGER (KIND=IK) :: LDay, NewHour, ResampTS                      ! More local integer variables
   INTEGER (KIND=IK) :: IN(7),N(7),Area, InUnits, NumSources         ! Distribution # array indices
   REAL    (KIND=RK) :: C0, C1, C, Out, ExpMR, Z, PPMFact            ! Local real variables
   REAL (KIND=RK), POINTER :: HV(:)                                  ! Pointers for distrib arguments    
   REAL    (KIND=RK) :: R, R2                                        ! Rand number for district choice
   REAL (KIND=RK),     ALLOCATABLE :: Temp(:,:,:)                    ! Temp array to handle mem problems
   ! Global constants: IK, LogU, LV_MS2, NO, PPM, RK, SL_FM2, ST_MS2, YES 
   ! Global vars used: AQData, DebugLevel, FirstProfile, InUnits, KSeed, NumDays, NumHours, 
   !                   NumMicros, NumMP, NumProfiles, NumSources, Person, PPMFact
   ! Global vars set : MicroConc, ProcLevel, ProcName
   ! Module constants: CS, ES, MASSBAL, VO
   ! Module vars used: DA, MPar, MPDistr, Mic, TS, CVVals
   ! Module vars set : Buff, CProd, CSum, EProd, ESum, MPV, NDist, NSamp, PP, RR, SS
   ! Intrinsic procs : ANY, CSHIFT, EXP, INT, MIN, MOD, RANDOM_SEED, WRITE,TRIM,ADJUSTL
   ! APEX procedures : Distrib, Fatal, EvaluateCondVars
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "MicroConcs"                                ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   ALLOCATE (Temp((-1*TimestepsPerDay+1):NumTS,-2:NumMP,NumLocs),   & 
      STAT=AllocErr)                                                 ! Temp array to handle local mem
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   InUnits    = Pol(PL)%InUnits                                      ! Input units for pollutant
   NumSources = Pol(PL)%NumSources                                   ! Num sources for pollutant  
   PPMFact    = Pol(PL)%PPMFact                                      ! Num sources for pollutant  
   MPV(:,1:,:)  = 0.                                                 ! Zero out micro concs
   ESum(:,:,:)  = 0.                                                 ! Zero out ESource sums
   CSum(:,:,:)  = 0.                                                 ! Zero out CSource sums
   EProd(:,:,:,:) = 1.                                               ! Set starting products to unity  
   CProd(:,:,:,:) = 1.                                               ! Set starting products to unity  
   LP(1) = Person(P)%HomeD                                           ! Place #1 is home district
   LP(2) = Person(P)%WorkD                                           ! Place #2 is work district
   LP(3) = 0                                                         ! Place #3 is zero
   IF (Pol(PL)%UseRoadAQ==YES) THEN                                  ! If using roadway AQs
      LP(4) = Person(P)%RoadD                                        ! Place #4 is either roadway district
      IF (Pol(PL)%RoadLast==YES) THEN                                ! If using work road AQs
         LP(7) = Person(P)%RoadWD                                    ! Place #7 is road work district
      ELSE                                                           ! Otherwise...
         LP(7) = 0                                                   ! It's zero
      ENDIF                                                          ! End road work conditional
   ELSE                                                              ! Or if it's not
      LP(4) = 0                                                      ! It's zero
   ENDIF                                                             ! End conditional   
   Step = 1                                                          ! Initialize time step counter
   AQData(PL,:,0) = 0                                                ! Init AQ average for person
   IF (SampleLocs==NO) THEN                                          ! If not customizing "O" location
      DO I=-1*TimestepsPerDay+1,NumTS                                ! Loop over all timesteps
         AQData(PL,I,0) = SUM(AQData(PL,I,1:))/NumDistricts          ! District zero is city average      
      ENDDO                                                          ! End loop over timesteps
   ELSE                                                              ! Otherwise, customize AQ
      DO I=1, NumODistricts                                          ! Loop over "other" districts
         CALL RANDOM_NUMBER(R)                                       ! Get uniform random
         DO K=1,NumDistricts                                         ! Loop over study districts
            Dis = K                                                  ! Update district number
         IF (R<(Real(K)/REAL(NumDistricts))) EXIT                    ! Exit if R < cumulative probab
      ENDDO                                                          ! End loop over districts
      IF (I==1. .AND. R<HomeProb) Dis=Person(P)%HomeD                ! If including home district      
      AQData(PL,:,0) = AQData(PL,:,0)                               &
         + AQData(PL,:,Dis)/NumODistricts                            ! Update "other" average
      ENDDO                                                          ! End loop over districts         
   ENDIF                                                             ! End "O" location AQ logic  
   IF (ResampleNear==YES) THEN                                       ! If resampling is indicated...
       DO H=-1*TimeStepsPerDay+1,NumTS                               ! Loop over TSteps in sim   
          CALL RANDOM_NUMBER(R)                                      ! Generate random number      
          IF (UseNHLocs==YES) THEN                                   ! If near home logic
             NHLocs(H) = NearbyS(Person(P)%HomeD,CEILING(R*NearbyS(  &
             Person(P)%HomeD,0)))                                    ! Random home loc
          ENDIF                                                      ! End near home logic
          IF (UseNWLocs==YES) THEN                                   ! If near work logic
              IF (Person(P)%WorkD .NE. -1) THEN                      ! If the person isn't a leaver
                 CALL RANDOM_NUMBER(R)                               ! Generate random number   
                 NWLocs(H) = NearbyS(Person(P)%WorkD,CEILING(R*   &
                    NearbyS(Person(P)%WorkD,0)))                     ! Random work loc               
              ELSE                                                   ! If the person is a leaver
                 NWLocs(H) = -1                                      ! Set leaver work location
              ENDIF                                                  ! End leaver logic
          ENDIF                                                      ! End near work logic
       ENDDO                                                         ! Loop over TS
   ELSE                                                              ! If no resampling...
       CALL RANDOM_NUMBER(R)                                         ! Generate random number      
       CALL RANDOM_NUMBER(R2)                                        ! Generate random number            
       DO H=-1*TimeStepsPerDay+1,NumTS                               ! Loop over TSteps in sim   
          IF (UseNHLocs==YES) THEN                                   ! If near home
             NHLocs(H) = NearbyS(Person(P)%HomeD,CEILING(R*       &
                NearbyS(Person(P)%HomeD,0)))                         ! Random home loc
          ENDIF                                                      ! End near home logic
          IF (UseNWLocs==YES) THEN                                   ! If near work...
             IF (Person(P)%WorkD .NE. -1) THEN                       ! If not a leaver
                NWLocs(H) = NearbyS(Person(P)%WorkD,CEILING(R2*   &
                   NearbyS(Person(P)%WorkD,0)))                      ! Random work loc          
             ELSE                                                    ! If the person is a leaver
                NWLocs(H) = -1                                       ! Set loc to -1
             ENDIF                                                   ! End leaver logic
          ENDIF                                                      ! End near work logic
       ENDDO                                                         ! End loop over timesteps
   ENDIF                                                             ! End reasmpling logic
   DO H=1,NumTS                                                      ! Loop over TSteps in sim   
      Name = "AQCONDITIONAL1"                                        ! Set AQCOND1 distribution name 
      IF (FuncNum(Name,NO)>0)  THEN                                  ! If AQCOND1 is defined 
         HV => AQData(PL,H:H,0)                                      ! Pointer to AQ                         
         CALL EvalFunc(Name=Name,R1=HV,IRes=AQCond(PL,H:H,0)%AQCond1)! Get AQCOND1 for current hour                          
      ENDIF                                                          ! End of AQCOND1 logic block
      Name = "AQCONDITIONAL2"                                        ! Set AQCOND2 distribution name 
      IF (FuncNum(Name,NO)>0)  THEN                                  ! If AQCOND2 is defined 
         HV => AQData(PL,H:H,0)                                      ! Pointer to AQ   
         CALL EvalFunc(Name=Name,R1=HV,IRes=AQCond(PL,H:H,0)%AQCond2)! Get AQCond2 for current hour                                         
      ENDIF                                                          ! End of AQCond2 logic block
      Name = "AQCONDITIONAL3"                                        ! Set AQCOND3 distribution name 
      IF (FuncNum(Name,NO)>0)  THEN                                  ! If AQCOND3 is defined 
         HV => AQData(PL,H:H,0)                                      ! Pointer to AQ                        
         CALL EvalFunc(Name=Name,R1=HV,IRes=AQCond(PL,H:H,0)%AQCond3)! Get AQCOND3 for current hour                                                 
      ENDIF                                                          ! End of AQCond3 logic block
      Name = "AQCONDITIONAL4"                                        ! Set AQCOND4 distribution name 
      IF (FuncNum(Name,NO)>0)  THEN                                  ! If AQCOND4 is defined 
         HV => AQData(PL,H:H,0)                                      ! Pointer to AQ                       
         CALL EvalFunc(Name=Name,R1=HV,IRes=AQCond(PL,H:H,0)%AQCond4)! Get AQCond4 for current hour                                               
      ENDIF                                                          ! End of AQCOND4 logic block
      Name = "AQCONDITIONAL5"                                        ! Set AQCOND5 distribution name 
      IF (FuncNum(Name,NO)>0)  THEN                                  ! If AQCOND5 is defined 
         HV => AQData(PL,H:H,0)                                      ! Pointer to AQ                          
         CALL EvalFunc(Name=Name,R1=HV,IRes=AQCond(PL,H:H,0)%AQCond5)! Get AQCond5 for current hour                                                     
      ENDIF                                                          ! End of AQCOND5 logic block
   ENDDO                                                             ! End loop over TSteps
   AQCond(PL,1-TimeStepsPerDay:0,:) = AQCond(PL,1:TimeStepsPerDay,:) ! Repeat first day of AQ data         
   DO MP=1, NumMP                                                    ! Loop over MP   
      IF (MPar(MP)%Pol .NE. PL .AND.                                &  
          MPar(MP)%Par .NE. VO .AND. MPar(MP)%Par .NE. AE)  CYCLE    ! MP not needed for pollut
      N   = MPar(MP)%N                                               ! N are indices for MP       
      M   = MPar(MP)%Mic                                             ! M is micro # for MP
      S   = MPar(MP)%Source                                          ! S is source # for MP 
      MPar(MP)%Seed = MPSeeds(MP)                                    ! Init MPar seed to global MPSeed 
      IF (MPar(MP)%CustomSeed > 0)  THEN                             ! If init seed given in micro file 
         MPar(MP)%Seed = MicroSeeds(P,MP)                            ! MP seed for current profile       
      ENDIF                                                          ! End custom MP seed logic
      CALL RANDOM_SEED(Put=MPar(MP)%Seed(1:KSeed))                   ! Initialize MP seed      
      DO L = 1,NumLocs                                                     ! Loop over places      
         IF (LP(2)<=0 .AND. L==2) CYCLE                              ! Cycle if leaver and work
         IF (LP(2)<=0 .AND. L==6) CYCLE                              ! Cycle if leaver and work
         IF (L==3) CYCLE                                             ! Cycle for 3 always         
         IF (L==4 .AND. Pol(PL)%UseRoadAQ==NO) CYCLE                 ! Cycle if roadway concs not needed
         IF (L==5 .AND. UseNHLocs==NO) CYCLE                         ! Cycle if near home not needed
         IF (L==6 .AND. UseNWLocs==NO) CYCLE                         ! Cycle if near work not needed
         IF (L==7 .AND. Pol(PL)%RoadLast==NO) CYCLE                  ! Cycle if road work not needed
         CALL EvaluateCondVars(P,MP,PL,L)                            ! Code is in MicroEnvModule 
         SampCount(:)   = 0                                          ! Initialize sample count to 0 
         NSamp(:,:)     = 1                                          ! Initialize sample numbers             
         LL = L                                                      ! Setting counter L
         IF (L==4) THEN                                              ! It on the road...                  
            Area   = DA(MP,Person(P)%RoadD)                          ! Area is roadway 
         ELSEIF (L==7) THEN                                          ! If on road at work...
            Area   = DA(MP,Person(P)%RoadWD)                         ! Area for this profile            
         ELSEIF (L==2) THEN                                          ! If at work...
            Area   = DA(MP,Person(P)%WorkD)                          ! Area for this profile
         ELSE                                                        ! Otherwise use home          
            Area   = DA(MP,Person(P)%HomeD)                          ! Area for this profile      
         ENDIF                                                       ! End of home/work logic
         IN(4)  = Area-1                                             ! Get the area index 
         ResampH  = MPar(MP)%ResH                                    ! Set hour resample flag
         ResampD  = MPar(MP)%ResD                                    ! Set day resample flag      
         ResampTS = MPar(MP)%ResTS                                   ! Set timestep resamp flag       
         DO I = 1, NumDays                                           ! Loop over simulation days
            IN(2) = MPar(MP)%WT(Calen(I)%DayofWeek)-1                ! Get the day index               
            IN(3) = MPar(MP)%MS(Calen(I)%Month)-1                    ! Get the season index
            DO TS = 1, TimeStepsPerDay                               ! Loop over steps in day
               T = (I-1)*TimeStepsPerDay+ TS                         ! Step number
               Hour = CEILING(REAL(TSMinutes*TS)/60.0)               ! Hour of day
               H    = (I-1)*24 + Hour                                ! Hour of simulation   
               IF (Hour .NE. LHour)  NewHour=YES                     ! First ts of new hour
               IF (I .NE. LDay)      NewDay(:)=YES                   ! 1st ts of new day, all dist#
               IN(1)  = MPar(MP)%HB(Hour)-1                          ! Get the block index
               IN(5)    = CVVals(1,H)-1                              ! Value of 1st CV          
               IN(6)    = CVVals(2,H)-1                              ! Value of 2nd CV
               IN(7)    = CVVals(3,H)-1                              ! Value of 3rd CV          
               DNum = IN(4)+N(4)*(IN(5)+N(5)*(IN(6)+N(6)*IN(7)))     ! Get distribution number
               DNum = 1+IN(1)+N(1)*(IN(2)+N(2)*(IN(3)+N(3)*DNum))    !   according to indices
               NDist(TS,I) = DNum                                    ! Set dist # for day/hour  
               IF (ResampD ==YES .AND. NewDay(DNum)==YES)  Resamp=1  ! Increment sample on new day
               IF (ResampH ==YES .AND. NewHour==YES) Resamp=1        ! Increment sample on new hour   
               IF (ResampTS==YES) Resamp=1                           ! Increment sample on new TS
               IF (SampCount(DNum)==0) Resamp = 1                    ! Always 1 samp, if distr used                   
               SampCount(DNum)=SampCount(DNum)+Resamp                ! Increment sample counter
               NSamp(TS,I) = SampCount(DNum)                         ! Set sample # for day/hour
               LHour = Hour                                          ! Set last hour index
               LDay  = I                                             ! Set last day index           
               NewHour = NO                                          ! Reset new hour flag   
               NewDay(DNum) = NO                                     ! Reset newday flag for dist#    
               Resamp = 0                                            ! Reset resamp flag
            ENDDO                                                    ! Continue with next hour
         ENDDO                                                       ! Continue with next day
         DO I = 1,MPar(MP)%NDistrib                                  ! Loop over distribs for this MP       
            DSS = SampCount(I)                                       ! # of samples needed from distr
            D   = MPDistr(I + MPar(MP)%ZDist)                        ! Set D to correct distrib
            IF (DSS>0) CALL SampleDist(D,Buff(I,1:DSS))              ! Get random values from distrib         
         ENDDO                                                       ! Continue with next distrib
         DO I=1,NumDays                                              ! Loop over simulation days
            DO TS = 1, TimeStepsPerDay                               ! Loop over steps in day
               Step = (I-1)*TimeStepsPerDay + TS                     ! Step number of simulation
               MPV(Step,MP,LL)  = Buff(NDist(TS,I),NSamp(TS,I))      ! Set MP value 
            ENDDO                                                    ! Continue with next hour
         ENDDO                                                       ! Continue with next day  
         IF (MPar(MP)%Par==ES) THEN                                  ! If MP is an emissions source...  
            IF (S==0) ESum(:,M,LL)    = ESum(:,M,LL)   +MPV(:,MP,LL) ! If source # = 0, add to ESum
            IF (S>0)  EProd(:,M,LL,S) = EProd(:,M,LL,S)*MPV(:,MP,LL) ! If source # > 0, multiply terms        
            IF (MPar(MP)%ResW==NO) THEN                              ! If ResW=NO... 
               IF (S==0) ESum(:,M,2)   =ESum(:,M,2)   +MPV(:,MP,1)   ! Add home source to Work sum
               IF (S>0)  EProd(:,M,2,S)=EProd(:,M,2,S)*MPV(:,MP,1)   ! Use home term in work product
            ENDIF                                                    ! End of work=home logic block 
         ENDIF                                                       ! End of ES logic block
         IF (MPar(MP)%Par==CS) THEN                                  ! If MP is a conc source
            IF (S==0) CSum(:,M,LL)    = CSum(:,M,LL)   +MPV(:,MP,LL) ! If source # = 0, add to CSum
            IF (S>0)  CProd(:,M,LL,S) = CProd(:,M,LL,S)*MPV(:,MP,LL) ! If source # > 0, multiply terms
            IF (MPar(MP)%ResW==NO) THEN                              ! If ResW=NO... 
               IF (S==0) CSum(:,M,2)   =CSum(:,M,2)   +MPV(:,MP,1)   ! Add home source to Work sum
               IF (S>0)  CProd(:,M,2,S)=CProd(:,M,2,S)*MPV(:,MP,1)   ! Use home term in work product
            ENDIF                                                    ! End of work=home logic block 
         ENDIF                                                       ! End of CS logic block 
      ENDDO                                                          ! Continue with next place
      IF(MPar(MP)%ResW==NO) MPV(:,MP,2) = MPV(:,MP,1)                ! Work = home if ResW=NO
      IF(MPar(MP)%Par==VO.AND.ANY(MPV(1:,MP,1:2)<=0.)) CALL Fatal(1) ! Stop if volume not positive 
   ENDDO                                                             ! Continue with next MP
   DO M=1,NumMicros                                                  ! Loop over micros  
      DO S=1,NumSources                                              ! Loop over numbered sources
         IF (NES(PL,M,S)>0) ESum(:,M,:) = ESum(:,M,:)+EProd(:,M,:,S) ! If ES then add product to ESum
         IF (NCS(PL,M,S)>0) CSum(:,M,:) = CSum(:,M,:)+CProd(:,M,:,S) ! If CS then add product to CSum
      ENDDO                                                          ! Continue with next source  
   ENDDO                                                             ! Continue with next micro
   DO MP = 1,NumMP                                                   ! Loop over MP
      IF (LP(2) <=0) THEN                                            ! If the person left the area
         MPV(:,MP,2) = MPV(:,MP,1)                                   ! Work MPV = Home MPV
         IF (UseNWLocs==YES) MPV(:,MP,6) = MPV(:,MP,1)               ! Near Work MPV = Home MPV  
      ENDIF                                                          ! End leaver logic
   ENDDO                                                             ! End MP loop
   MPV(:,1:,3)     = 0.5* (MPV(:,1:,1) + MPV(:,1:,2))                ! Place #3 is average of #1 & #2
   ESum(:,:,3)     = 0.5* (ESum(:,:,1) + ESum(:,:,2))                ! Same for ES sums
   CSum(:,:,3)     = 0.5* (CSum(:,:,1) + CSum(:,:,2))                ! Same for CS sums
   ESum((-1*TimestepsPerDay+1):0,:,:) = ESum(1:TimeStepsPerDay,:,:)  ! Same for ES sums
   CSum((-1*TimestepsPerDay+1):0,:,:) = CSum(1:TimeStepsPerDay,:,:)  ! Same for CS sums
   MPV((-1*TimestepsPerDay+1):0,1:,:) = MPV(1:TimeStepsPerDay,1:,:)  ! First day is repeated 
   IF (InUnits==PPM .OR. InUnits==PPB)                              &
      ESum(:,:,:) = ESum(:,:,:)/PPMFact                              ! Convert ug/hr to ppm*m3/hr
   IF (ANY(ESum<0.)) CALL Fatal(2)                                   ! Stop if negative ESources
   IF (ANY(CSum<0.)) CALL Warning(1)                                 ! Warn if negative CSources
   DO M=1,NumMicros                                                  ! Loop over micros
      IF (Mic(PL,M)%PRX>0) THEN                                      ! If MP defined
         if (ANY(MPV(:,Mic(PL,M)%PRX,:)<0)) CALL Fatal(4)            ! It must be positive                                   ! Stop if negative MP value           
      ENDIF                                                          ! End MP logic
      IF (Mic(PL,M)%PEN>0) THEN                                      ! If MP defined
         if (ANY(MPV(:,Mic(PL,M)%PEN,:)<0)) CALL Fatal(4)            ! It must be positive                                   ! Stop if negative MP value     
      ENDIF                                                          ! End MP logic
      IF (Mic(PL,M)%AER>0) THEN                                      ! If MP defined
         if (ANY(MPV(:,Mic(PL,M)%AER,:)<0)) CALL Fatal(4)            ! It must be positive                                   ! Stop if negative MP value             
      ENDIF                                                          ! End MP logic
      IF (Mic(PL,M)%DEC>0) THEN                                      ! If MP defined
         if (ANY(MPV(:,Mic(PL,M)%DEC,:)<0)) CALL Fatal(4)            ! It must be positive                                   ! Stop if negative MP value     
      ENDIF                                                          ! End MP logic
      IF (Mic(PL,M)%VOL>0) THEN                                      ! If MP defined  
         if (ANY(MPV(:,Mic(PL,M)%VOL,:)<0)) CALL Fatal(4)            ! It must be positive                                   ! Stop if negative MP value         
      ENDIF                                                          ! End MP logic    
      IF (Mic(PL,M)%MR>0) THEN                                       ! If MP defined     
         if (ANY(MPV(:,Mic(PL,M)%MR,:)<0)) CALL Fatal(4)             ! It must be positive                                   ! Stop if negative MP value               
      ENDIF                                                          ! End MP logic
      PP(:,:) = MPV(:,Mic(PL,M)%Prx,:) * MPV(:,Mic(PL,M)%Pen,:)      ! Product of proximity and penetr
      IF (Mic(PL,M)%Method==MASSBAL) THEN                            ! For MASSBAL micros...       
         Temp(:,-2,:) =MPV(:,Mic(PL,M)%AER,:)+MPV(:,Mic(PL,M)%Dec,:) ! Find default hourly removal rate
         MPV(:,-2,:)  =Temp(:,-2,:)                                  ! Find default hourly removal rate     
         DO L=1,NumLocs                                              ! Loop over places         
            IF (L==4 .AND. Pol(PL)%UseRoadAQ==NO) CYCLE              ! Cycle if roadway concs not needed
            IF (L==5 .AND. UseNHLocs==NO) CYCLE                      ! Cycle if near home not needed
            IF (L==6 .AND. UseNWLocs==NO) CYCLE                      ! Cycle if near work not needed
            IF (L==7 .AND. Pol(PL)%RoadLast==NO) CYCLE               ! Cycle if road work not needed
            RR(:) = MPV(:,Mic(PL,M)%MR,L)                            ! Set CSum removal rates          
            SS(:) = 0.                                               ! Initialize sources to zero
            IF (Mic(PL,M)%NES>0)                                    &
               SS(:)=ESum(:,M,L)/MPV(:,Mic(PL,M)%Vol,L)              ! Add to SS if ESources exist
            IF (Mic(PL,M)%NCS>0) SS(:)=SS(:) + CSum(:,M,L)*RR(:)     ! Add to SS if CSources exist
            RR(:) = MPV(:,-2,L)                                      ! Removal = AirEx + Decay
            C0 = 0.                                                  ! Start with nothing in micro                   
            DO J=(-1*TimestepsPerDay+1), NumTS                       ! Loop over all timesteps
               IF(L==5) LP(L) = SList(NHLocs(J))%NumD                ! Set random near home loc  
               IF (L==6) THEN                                        ! If near work loc
                   IF(NWLocs(J)/=-1) LP(L)=SList(NWLocs(J))%NumD     ! If not a leaver
                   IF(NWLocs(J)==-1) LP(L)=-1                        ! If a leaver...
               ENDIF                                                 ! End near work logic
               Z = MIN(RR(J)*REAL(TSMinutes)/60.0,60.)               ! Numerical limit for exp(-RR)
               IF (Z==0.) THEN                                       ! If no removal...
                  C1 = C0 + SS(J)                                    ! Conc increases by source amount                   
                  MicroConc(J,M,L) = C0 + SS(J)*0.5                  ! Mean conc is start-end average
               ELSE                                                  ! Else removal occurs...
                  ExpMR = EXP(-Z)                                    ! Call EXP once, value used twice    
                  Out =AQData(PL,J,LP(L))                           &
                     *PP(J,L)*MPV(J,Mic(PL,M)%AER,L)                 ! Contribution from outside air               
                  C = (SS(J)+Out)/RR(J)                              ! Equilibrium concentration                
                  C1 = C + (C0-C)*ExpMR                              ! End-of-hour concentration             
                  IF ((1.0-ExpMR)/RR(J) > 1.0 .AND. RR(J) > 0) THEN  ! If a rounding error occurs ... 
                     MicroConc(J,M,L) = C +                         &
                     (C0-C)*REAL(TSMinutes)/60.0                     ! Do Special Average conc. over hour 
                  ELSE                                               ! Else normal conc. occurs 
                     MicroConc(J,M,L) = C +                         &
                     (C0-C)*(1.0-ExpMR)/(RR(J)*REAL(TSMinutes)/60.0) ! Average concentration over hour              
                  ENDIF                                              ! End of conc calculation   
               ENDIF                                                 ! End of current hour calculations
               C0 = C1                                               ! Final conc becomes initial 
            ENDDO                                                    ! Continue with next hour
         ENDDO                                                       ! Continue with next place
      ELSE                                                           ! Else a FACTORS micro...
         DO L=1,NumLocs                                              ! Loop over places
            IF (L==5 .AND. UseNHLocs==NO) CYCLE                      ! Ignore if not needed
            IF (L==6 .AND. UseNWLocs==NO) CYCLE                      ! Ignore if not needed         
            DO J=(-1*TimestepsPerDay+1), NumTS                       ! Loop over all timesteps
               IF(L==5) LP(L) = SList(NHLocs(J))%NumD                ! Set random near home loc
               IF(L==6) THEN                                         ! If near work loc
                  IF (NWLocs(J)/=-1) LP(L)=SList(NWLocs(J))%NumD     ! If not a leaver
                  IF (NWLocs(J)==-1) LP(L) = -1                      ! If a leaver...           
               ENDIF                                                 ! End near work logic
               MicroConc(J,M,L)=AQData(PL,J,LP(L))*PP(J,L)+CSum(J,M,L)! FACTORS equation for micro conc         
            ENDDO                                                    ! End loop over time steps
         ENDDO                                                       ! Continue with next place
      ENDIF                                                          ! End of method logic   
      IF (NumLocs>=4 .AND. Pol(PL)%UseRoadAQ==NO) MicroConc(:,M,4)=0 ! Set to zero if not needed
      IF (NumLocs>=5 .AND. UseNHLocs==NO) MicroConc(:,M,5) = 0       ! Set to zero if not needed
      IF (NumLocs>=6 .AND. UseNWLocs==NO) MicroConc(:,M,6) = 0       ! Set to zero if not needed
      IF (NumLocs>=7 .AND. Pol(PL)%RoadLast==NO) MicroConc(:,M,7) =0 ! Set to zero if not needed
      IF (ANY(MicroConc(:,M,:)<0.)) THEN                             ! If a negative micro conc found
         CALL Warning(2)                                             ! Warn if negative MicroConcs          
         DO J=(-1*TimestepsPerDay+1), NumTS                          ! Loop over timesteps
            DO L=1,7                                                 ! Loop over places
               IF (MicroConc(J,M,L)<0.) THEN                         ! If conc is negative ...
                  WRITE(LogU,*) 'Neg Conc ',J,M,L,MicroConc(J,M,L)   ! Record hour, micro, place, value 
                  WRITE(LogU,*) 'AirQual= ',AQData(PL,J,LP(L))       ! Put AQ data on log 
                  WRITE(LogU,*) 'PP     = ',PP(J,L)                  ! Put Prox*pen on log
                  WRITE(LogU,*) 'CSum   = ',CSum(J,M,L)              ! Put CSum on log
                  IF (Mic(PL,M)%Method==MASSBAL) THEN                ! If a MASSBAL micro
                     WRITE(LogU,*) 'Source =',SS(J)                  ! Put Source on log
                     WRITE(LogU,*) 'AER    =',MPV(J,Mic(PL,M)%AER,L) ! Put AER on log
                     WRITE(LogU,*) 'Removal=',RR(J)                  ! Put removal rate on log
                  ENDIF                                              ! End of MASSBAL logic
                  MicroConc(J,M,L) = 0                               ! Set negative MicroConc to zero                  
               ENDIF                                                 ! End of negative conc block
            ENDDO                                                    ! Continue with next place
         ENDDO                                                       ! Continue with next hour
         IF (Mic(PL,M)%Method==MASSBAL) THEN                         ! If a MASSBAL micro
            CALL Fatal(5)                                            ! If MicroConc <0 in MASSBAL, Fatal
         ENDIF                                                       ! End MASSBAL Fatal call
      ENDIF                                                          ! End of negative conc logic
   ENDDO                                                             ! Continue with next micro
   IF (MResOut==YES) THEN                                            ! If printing micro results   
      CALL WriteMResFile(P,PL)                                       ! Write to MRes file
   ENDIF                                                             ! End file-writing logic
   DEALLOCATE (Temp)                                                 ! Deallocate internal arrays
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE MicroConcs                                            ! Return to main APEX program


SUBROUTINE EvaluateCondVars(P,MP,PL,L)                               ! Called from MicroConcs
   INTEGER (KIND=IK), INTENT(IN)  :: P                               ! Input argument is profile #
   INTEGER (KIND=IK), INTENT(IN)  :: MP                              ! Input argument is MP #
   INTEGER (KIND=IK), INTENT(IN)  :: PL                              ! Input argument is Pollutant #   
   INTEGER (KIND=IK), INTENT(IN)  :: L                               ! Input argument is location #
   INTEGER (KIND=IK) :: I, J, K                                      ! More local integer variables
   ! Global constants: 
   ! Global vars used: DebugLevel, NumDays
   ! Global vars set : ProcLevel, ProcName
   ! Module vars used: MPar
   ! Module vars set : CVVals
   ! Intrinsic procs : WRITE
   ! APEX procedures : CVValue
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "EvaluateCondVars"                          ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) ST_MS3, ProcName(ProcLevel)  ! Message for debugging
   CVVals(:,:)    = 1                                                ! Default CV value is 1 
   DO I=1, NumDays                                                   ! Loop over days
      DO J=1,3                                                       ! Loop over CV
        IF (MPar(MP)%C(J)==0) CYCLE                                  ! Continue if CV not used
         IF (CVList(MPar(MP)%C(J))%Rate==HOURLY) THEN                ! If the CV is an hourly CV
            DO K=1,24                                                ! Loop over hours
               CVVals(J,K+24*(I-1)) =                               &  
                  CVValue(P,K+24*(I-1),MPar(MP)%C(J),PL,L)           ! Get the hourly CV values
            ENDDO                                                    ! End of hour loop      
         ELSEIF (CVList(MPar(MP)%C(J))%Rate==TStep) THEN             ! If the CV is an hourly CV
            DO K=1,TimeStepsPerDay                                   ! Loop over hours 
               CVVals(J,K+TimeStepsPerDay*(I-1)) =                  &  
                  CVValue(P,K+TimeStepsPerDay*(I-1),MPar(MP)%C(J),PL,L)! Get the AQ TSstep CV values
            ENDDO                                                    ! End of hour loop                     
         ELSE                                                        ! Otherwise if daily CV           
            CVVals(J,24*(I-1)+1:24*(I-1)+24) =                      &
               CVValue(P,I,MPar(MP)%C(J),PL,L)                       ! All hours get daily val
         ENDIF                                                       ! End rate-of-change logic                                             
      ENDDO                                                          ! Continue with next CV
   ENDDO                                                             ! End loop over days   
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) LV_MS3, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE EvaluateCondVars                                      ! Return to MicroConcs

SUBROUTINE WriteMResFile(P, PL)                                      ! Called from Output
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input parameters
   INTEGER (KIND=IK) :: I, J, L, MM, M, NVars, NLoc                  ! Local integer variables
   INTEGER (KIND=IK) :: LP(7), D, H, TS, Hr, T, Step                 ! Local integer variables
   CHARACTER (LEN=20)  :: MHeader(NUMMRESVARS)                       ! MRes file var header
   CHARACTER (LEN=20)  :: MVals(NUMMRESVARS)                         ! MRes file var values   
   LOGICAL (KIND=1)    :: F(NUMMRESVARS)                             ! Short form of psum flags
   ! Global constants: HourU
   ! Global vars used: NumMicrosPr, MList, Mic, MPV, CSum, ESum,
   !                   AQData, MicroConc, SS, MPV           
   ! Global vars set : ProcLevel, ProcName   
   ! Intrinsic procs : WRITE, SIZE, ALLOCATE, TRIM, ADJUSTL, ADJUSTR, ALLOCATED
   ! APEX procs      : CheckAllocation
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteMResFile"                             ! Procedure name for messages
   NVars=NUMMRESVARS                                                 ! Number of variables to write
   F=(MResFlags==YES)                                                ! Set F array to True/False
   LP(1) = Person(P)%HomeD                                           ! Place #1 is home district
   LP(2) = Person(P)%WorkD                                           ! Place #2 is work district
   LP(3) = 0                                                         ! Place #3 is zero   
   LP(4) = Person(P)%RoadD                                           ! Place #4 is road district 
   LP(7) = Person(P)%RoadWD                                          ! Place #7 is road work dist.
   IF (P==FirstProfile) THEN                                         ! If first person
      MHeader= ""                                                    ! Initialize the header array
      MVals  = ""                                                    ! Initialize the values array
      WRITE(MHeader(1),'(A7)') 'Person'                              ! Write label to header array  
      WRITE(MHeader(2),'(A6)') 'Micro'                               ! Write label to header array
      WRITE(MHeader(3),'(A4)') 'Loc'                                 ! Write label to header array
      WRITE(MHeader(4),'(A6)') 'Hour'                                ! Write label to header array       
      WRITE(MHeader(5),'(A6)') 'Step'                                ! Write label to header array       
      IF(F(6)) WRITE(MHeader(6),'(A4)') 'Prx'                        ! Write label to header array
      IF(F(7)) WRITE(MHeader(7),'(A4)') 'Pen'                        ! Write label to header array
      IF(F(8)) WRITE(MHeader(8),'(A5)') 'CSum'                       ! Write label to header array
      IF(F(9)) WRITE(MHeader(9),'(A4)') 'Amb'                        ! Write label to header array
      IF(F(10)) WRITE(MHeader(10),'(A5)') 'Conc'                     ! Write label to header array
      IF(F(11)) WRITE(MHeader(11),'(A5)') 'ESum'                     ! Write label to header array
      IF(F(12)) WRITE(MHeader(12),'(A7)') 'Source'                   ! Write label to header array
      IF(F(13)) WRITE(MHeader(13),'(A7)') 'Volume'                   ! Write label to header array
      IF(F(14)) WRITE(MHeader(14),'(A7)') 'Decay'                    ! Write label to header array
      IF(F(15)) WRITE(MHeader(15),'(A4)') 'AER'                      ! Write label to header array
      IF(F(16)) WRITE(MHeader(16),'(A3)') 'RR'                       ! Write label to header array
      IF(F(17)) WRITE(MHeader(17),'(A10)') 'WindowRes'               ! Write label to header array
      IF(F(18)) WRITE(MHeader(18),'(A10)') 'WindowCar'               ! Write label to header array
      IF(F(19)) WRITE(MHeader(19),'(A11)') 'MaxTempCat'              ! Write label to header array
      IF(F(20)) WRITE(MHeader(20),'(A11)') 'AvgTempCat'              ! Write label to header array
      IF(F(21)) WRITE(MHeader(21),'(A9)') 'SpeedCat'                 ! Write label to header array
      IF(F(22)) WRITE(MHeader(22),'(A6)') 'Cond1'                    ! Write label to header array
      IF(F(23)) WRITE(MHeader(23),'(A6)') 'Cond2'                    ! Write label to header array
      IF(F(24)) WRITE(MHeader(24),'(A6)') 'Cond3'                    ! Write label to header array
      IF(F(25)) WRITE(MHeader(25),'(A8)') 'TempCat'                  ! Write label to header array
      IF(F(26)) WRITE(MHeader(26),'(A9)') 'HumidCat'                 ! Write label to header array
      IF(F(27)) WRITE(MHeader(27),'(A10)')'PrecipCat'                ! Write label to header array
      IF(F(28)) WRITE(MHeader(28),'(A8)') 'WindCat'                  ! Write label to header array
      IF(F(29)) WRITE(MHeader(29),'(A7)') 'DirCat'                   ! Write label to header array
      IF(F(30)) WRITE(MHeader(30),'(A4)') 'Day'                      ! Write label to header array
      IF(F(31)) WRITE(MHeader(31),'(A6)') 'Month'                    ! Write label to header array
      IF(F(32)) WRITE(MHeader(32),'(A9)') 'District'                 ! Write label to header array
      IF(F(33)) WRITE(MHeader(33),'(A9)') 'DayWeek'                  ! Write label to header array
      IF(F(34)) WRITE(MHeader(34),'(A9)') 'AQCond1'                  ! Write label to header array      
      IF(F(35)) WRITE(MHeader(35),'(A9)') 'AQCond2'                  ! Write label to header array  
      IF(F(36)) WRITE(MHeader(36),'(A9)') 'AQCond3'                  ! Write label to header array  
      IF(F(37)) WRITE(MHeader(37),'(A9)') 'AQCond4'                  ! Write label to header array  
      IF(F(38)) WRITE(MHeader(38),'(A9)') 'AQCond5'                  ! Write label to header array                          
      WRITE(Pol(PL)%MResU,'(100A)') (TRIM(MHeader(I)), I=1,NVars)    ! Write final header to file
   ENDIF                                                             ! End first profile logic
   IF (MResHome==YES) THEN                                           ! If only printing home sector 
       NLoc=1                                                        ! one location, home sec 
   ELSE                                                              ! Otherwise...
       NLoc=3                                                        ! Print all three
       IF (Pol(PL)%UseRoadAQ==YES) NLoc=4                            ! If roadway used, print
       IF (UseNHLocs==YES) NLoc=5                                    ! If near home used, print
       IF (UseNWLocs==YES) NLoc=6                                    ! If near work used, print
       IF (Pol(PL)%RoadLast==YES) NLoc=7                             ! If road work used, print       
   ENDIF                                                             ! End print logic
   DO MM=1, NumMicrosPr                                              ! Loop over micros to print
      M = MList(MM)                                                  ! Current micro to write
      IF (MList(MM)>NumMicros) THEN                                  ! Out-of-range micro number
         CYCLE                                                       ! Continue with next micro
      ENDIF                                                          ! End bad micro logic
      Step = 1                                                       ! Initialize time step counter
      DO L=1,NLoc                                                    ! Loop over locations
         IF (L==4 .AND. Pol(PL)%UseRoadAQ==NO) CYCLE                 ! Don't print if not asked for
         IF (L==5 .AND. UseNHLocs==NO) CYCLE                         ! Don't print if not asked for
         IF (L==6 .AND. UseNWLocs==NO) CYCLE                         ! Don't print if not asked for
         IF (L==7 .AND. Pol(PL)%RoadLast==NO) CYCLE                  ! Don't print if not asked for
         DO  J=(-1*TimestepsPerDay+1),NumTS                          ! Loop over timesteps 
            IF (UseNHLocs==YES) THEN                                 ! If using near home locs
               LP(5) = SList(NHLocs(J))%NumD                         ! Set random near home loc
            ELSE                                                     ! Else if not using them
               LP(5) = 0                                             ! Set value to 0
            ENDIF                                                    ! End near home logic
            IF (UseNWLocs==YES) THEN                                 ! If using near work locs
                IF (NWLocs(J)/=-1) THEN                              ! If not a leaver...
                   LP(6) = SList(NWLocs(J))%NumD                     ! Set random near work loc   
                ELSE                                                 ! Else if a leaver
                   LP(6) = -1                                        ! Set to outside loc
                ENDIF                                                ! End leaver logic
            ELSE                                                     ! If not using near work locs
               LP(6) = 0                                             ! Set value to 9
            ENDIF                                                    ! End near work logic
            D  =  FLOOR(REAL(J-1)/REAL(TimeStepsPerDay))+1           ! Day number for this hour   
            T  =  MAX(J,1)                                           ! Positive version of timestep
            TS =  J-(D-1)*TimeStepsPerDay                            ! Timestep number
            IF (J<=0) THEN                                           ! When hour is negative   
               TS=J                                                  ! Set 0-24 hour to neg val  
               D = 1                                                 ! Only happens on day 1
            ENDIF                                                    ! End negative hour logic   
            Hr =  CEILING(REAL(TS)/TimeStepsPerHour)                 ! Hour number
            H  =  MAX(Hr,1)                                          ! Positive version of hour
            IF (TS<=0) Hr=                                          &   
               -1*FLOOR(REAL(-1*TS)/TimeStepsPerHour)                ! Hour number for prior day
            WRITE(MVals(1),'(I7)') P                                 ! Write var to val array
            WRITE(MVals(2),'(I3)') M                                 ! Write var to val array
            WRITE(MVals(3),'(I2)') L                                 ! Write var to val array
            WRITE(MVals(4),'(I5)') Hr                                ! Write var to val array
            WRITE(MVals(5),'(I5)') TS                                ! Write var to val array
            IF (F(6)) WRITE(MVals(6),'(F7.4)')                      &
               MPV(J,Mic(PL,M)%Prx,L)                                ! Write var to val array
            IF (F(7)) WRITE(MVals(7),'(F7.4)')                      &
               MPV(J,Mic(PL,M)%Pen,L)                                ! Write var to val array
            IF (F(8)) WRITE(MVals(8),'(ES10.3)') CSum(J,M,L)         ! Write var to val array
            IF (F(9)) WRITE(MVals(9),'(ES10.3)') AQData(PL,J,LP(L))  ! Write var to val array
            IF (F(10)) WRITE(MVals(10),'(ES10.3)') MicroConc(J,M,L)  ! Write var to val array
            IF (Mic(PL,M)%Method==MASSBAL)  THEN                     ! If MASSBAL micro   
               IF (F(11)) WRITE(MVals(11), '(ES10.3)') ESum(J,M,L)   ! Write var to val array 
               IF (F(12)) WRITE(MVals(12), '(ES10.3)') SS(J)         ! Write var to val array 
               IF (Mic(PL,M)%VOL > 0.AND.Mic(PL,M)%VOL < NumMP) THEN ! If volume present for mic
                  IF (F(13))  WRITE(MVals(13),'(ES11.4)')           &
                     MPV(J,Mic(PL,M)%VOL,L)                          ! Write var to val array
                  ELSE                                               ! Otherwise vol not defined
                     IF (F(13))  WRITE(MVals(13),'(I11)') 0          ! Pad with 0             
               ENDIF                                                 ! End volume logic
               IF (Mic(PL,M)%DEC > 0.AND.Mic(PL,M)%DEC < NumMP) THEN ! If decay present for mic
                  IF (F(14))  WRITE(MVals(14),'(ES11.4)')           &
                     MPV(J,Mic(PL,M)%DEC,L)                          ! Write var to val array
                  ELSE                                               ! Otherwise dec not defined
                     IF (F(14))  WRITE(MVals(14),'(I11)') 0          ! Pad with 0             
               ENDIF                                                 ! End volume logic
               IF (F(15)) WRITE(MVals(15),'(F8.4)')                 &
                  MPV(J,Mic(PL,M)%AER,L)                             ! Write var to val array
               IF (F(16)) WRITE(MVals(16),'(F8.4)') RR(J)            ! Write var to val array
            ELSE                                                     ! Otherwise micro is FACTORS
               IF (F(11))  WRITE(MVals(11), '(I10)') 0               ! Pad with 0
               IF (F(12))  WRITE(MVals(12), '(I10)') 0               ! Pad with 0
               IF (F(13))  WRITE(MVals(13), '(I11)') 0               ! Pad with 0
               IF (F(14))  WRITE(MVals(14), '(I8)') 0                ! Pad with 0
               IF (F(15))  WRITE(MVals(15), '(I8)') 0                ! Pad with 0
               IF (F(16))  WRITE(MVals(16), '(I8)') 0                ! Pad with 0
            ENDIF                                                    ! End MASSBAL logic 
               IF (F(17))  WRITE(MVals(17), '(I2)') CVValue(P,D,13,PL,L) ! Write WindowRes
               IF (F(18))  WRITE(MVals(18), '(I2)') CVValue(P,D,14,PL,L) ! Write WindowCar
               IF (F(19))  WRITE(MVals(19), '(I2)') CVValue(P,D,15,PL,L) ! Write MaxTempCat
               IF (F(20))  WRITE(MVals(20), '(I2)') CVValue(P,D,16,PL,L) ! Write AvgTempCat
               IF (F(21))  WRITE(MVals(21), '(I2)') CVValue(P,D,17,PL,L) ! Write SpeedCat
               IF (F(22))  WRITE(MVals(22), '(I2)') CVValue(P,D,18,PL,L) ! Write DailyCond1
               IF (F(23))  WRITE(MVals(23), '(I2)') CVValue(P,D,19,PL,L) ! Write DailyCond2
               IF (F(24))  WRITE(MVals(24), '(I2)') CVValue(P,D,20,PL,L) ! Write DailyCond3
               IF (F(25))  WRITE(MVals(25), '(I2)') CVValue(P,H,21,PL,L) ! Write TempCat
               IF (F(26))  WRITE(MVals(26), '(I2)') CVValue(P,H,22,PL,L) ! Write HumidCat
               IF (F(27))  WRITE(MVals(27), '(I2)') CVValue(P,H,23,PL,L) ! Write PrecipCat
               IF (F(28))  WRITE(MVals(28), '(I2)') CVValue(P,H,24,PL,L) ! Write WindCat
               IF (F(29))  WRITE(MVals(29), '(I2)') CVValue(P,H,25,PL,L) ! Write DirCat    
               IF (F(30)) WRITE(MVals(30),'(I4)') D                  ! Write day of sim
               IF (F(31)) WRITE(MVals(31),'(I3)') Calen(D)%Month     ! Write month 
               IF (F(32)) THEN                                       ! If writing district 
                  IF (L==1) WRITE(MVals(32),'(I6)') Person(P)%HomeD  ! Write air district 
                  IF (L==2) WRITE(MVals(32),'(I6)') Person(P)%WorkD  ! Write air district 
                  IF (L==3) WRITE(MVals(32),'(I6)') 0                ! District 0 is city mean 
                  IF (L==4) WRITE(MVals(32),'(I6)') Person(P)%RoadD  ! Write roadway district
                  IF (L==5) THEN                                     ! If near home district
                      WRITE(MVals(32),'(I6)') NHLocs(TS)             ! Write near home district           
                      Step = Step + 1                                ! Increment time step
                  ENDIF                                              ! End near home logic
                  IF (L==6) THEN                                     ! If near work district
                      WRITE(MVals(32),'(I6)') NWLocs(TS)             ! Write near work district
                      Step= Step + 1                                 ! Increment time step
                  ENDIF                                              ! End near work logic                  
                  IF (L==7) WRITE(MVals(32),'(I6)') Person(P)%RoadWD ! Write roadway district                  
               ENDIF                                                 ! End district logic
               IF (F(33)) WRITE(MVals(33),'(I2)') Calen(D)%DayofWeek ! Write day of week
               IF (F(34)) WRITE(MVals(34),'(I2)') CVValue(P,J,31,PL,L) ! Write AQCond1   
               IF (F(35)) WRITE(MVals(35),'(I2)') CVValue(P,J,32,PL,L) ! Write AQCond2  
               IF (F(36)) WRITE(MVals(36),'(I2)') CVValue(P,J,33,PL,L) ! Write AQCond3  
               IF (F(37)) WRITE(MVals(37),'(I2)') CVValue(P,J,34,PL,L) ! Write AQCond4  
               IF (F(38)) WRITE(MVals(38),'(I2)') CVValue(P,J,35,PL,L) ! Write AQCond5                                                                              
            WRITE(Pol(PL)%MResU,*)                                  &
            (TRIM(MVals(I)),I=1,NVars)                               ! Write final value array to file
         ENDDO                                                       ! End loop over hours
      ENDDO                                                          ! End loop over location
   ENDDO                                                             ! End loop over micros 
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
 END SUBROUTINE WriteMResFile                                        ! Return to MicroConcs 

END MODULE MicroEnvModule                                            ! End of source code for module

